// FILENAME:	MI_Mars_3_SandD.cs
//
//------------------------------------------------------------------------------

$missionName = "Mars, Gamma Sector";  // DM_Mars_Battle
exec("MercsIncMultiplayerStdLib.cs");
exec("MIConfigCosts.cs");

$MissionCycling::Stage0 = "MercsInc";

function setDefaultMissionOptions()
{
  $server::TeamPlay = true;
  $server::AllowDeathmatch = false;
  $server::AllowTeamPlay = true;	
	
  $server::AllowTeamRed = true;
  $server::AllowTeamBlue = false;
  $server::AllowTeamYellow = false;
  $server::AllowTeamPurple = false;
}


function onMissionStart()
{
  MarsSounds();
}


function onMissionLoad()
{
  $MIstateOfMission = 0;  // 0 = not started, 1 = in progress, 2 = victory, 3 = loss
  $MIinGamePlayers = 0;
  if($MItransportPercent == "") $MItransportPercent = 1;
  if($MIbountyPercent == "") $MIbountyPercent = 1;
  if($MItotalPay == "") $MItotalPay = 800;

  // Mission parameters 
  // 0 = number of targets
  // 1 = types of targets
  if($MImissionParameter[0] == "") $MImissionParameter[0] = 1;
  if($MImissionParameter[1] == "") $MImissionParameter[1] = 1;

  %count = (playerManager::getPlayerCount());
  for(%i = 0; %i < %count; %i++)
  {
    %player = playerManager::getPlayerNum(%i);
    %player.MIdied = 0;
    %player.MIstartMission = 0;
  }
}


//======================


function player::onAdd(%player)
{
  %player.MIdied = 0;
  %player.MIloggedIn = 0;
  %player.MIstartMission = 0;
}


function player::onRemove(%player)
{
  %count = (playerManager::getPlayerCount() - 1);
  if(%count == 0)
  {
    $MIstateOfMission = 3;
    schedule("missionEndConditionMet();", 1);
  }
  if( $MIstateOfMission == 1 ) return;
  else Mercs::startCheck();
}


function vehicle::OnAdd(%vehicle)
{
  %player = playermanager::vehicleIDtoPlayerNum(%vehicle);
  if(%player == 0) return;
  setTeam(%vehicle, *IDSTR_TEAM_RED);

  if(%player.MIdied == 1)
  {
    damageobject(%vehicle, 50000);
    say(%player, 0, "You have already died.  Please wait for the mission to complete (your bonus will be forfeit if you quit early.");
    return;
  }

  if($MIstateOfMission != 0)
  {
    say(%player, 0, "You cannot join a mission in progress.  Please wait until the mission ends.");
    damageobject(%vehicle, 50000);
    return;
  }

  if(%player.MIloggedIn != 1)
  {
    damageobject(%vehicle, 50000);
    messagebox(%player, "Sorry, this mission has already launched.  Please wait until the battlegroup returns to the lobby.  Thank you for your patience.");
  }
  else
  {
    redrop(%vehicle);
    %vehicleCost = MIConfigCosts::checkvalue(%vehicle);
    %player.MIvehicleCost = %vehicleCost;
    if(%vehicleCost > (%player.MImoney + 1) )
    {
      messagebox(%player, "Your vehicle cost is $" @ %vehicleCost @ ".  You only have $" @ %player.MImoney @ " in your account.  Please select a less expensive vehicle.");
      damageobject(%vehicle, 50000);
    }
    else
    {
      %player.MItransportCost = Mercs::round(%vehicleCost * $MItransportPercent / 100);
      messagebox(%player, "Your vehicle cost is $" @ %vehicleCost @ ".  Transport fees will be $" @ %player.MItransportCost @ ".  Please log in again to confirm your selection.");
      Mercs::keepVehicleShutdown(%vehicle);
      %player.MIdied = 0;
    }
  }
}


function vehicle::OnDestroyed(%target, %attacker)
{
  schedule("deleteObject(" @ %target @ ");", 1);

  if($MIstateOfMission == 0) return;

  %attackerPlayer = playerManager::vehicleIdToPlayerNum(%attacker);
  %targetPlayer = playerManager::vehicleIdToPlayerNum(%target);

  if(%targetPlayer != 0)
  {
    if(%targetPlayer.MIstartMission != 1)
    {
      return;
    }
    else
    {
      say(0, 0, getName(%targetPlayer) @ " is down");
      %player.MIdied = 1;
      %targetPlayer.MIstartMission = 0;
      $MIinGamePlayers--;
      say(%targetPlayer, 0, "$" @ %targetPlayer.MIvehicleCost @ " fee for losing your vehicle.  Please remain in the vicinity until mission end to collect your bonus.");
      %fee = %targetPlayer.MIvehicleCost * -1;
      %targetPlayer.MItransportCost = 0;
      Mercs::addMoney(%targetPlayer, %fee);
      %targetPlayer.MIlostVehicle = 1;
    }
  }

  else
  {
    if(%attackerPlayer == 0) return;

    %points = Mercs::round(MIConfigCosts::checkValue(%target) * $MIbountyPercent / 100);

    if(%target != %attacker)
    {
      %attackerPlayer.MIbounty = %attackerPlayer.MIbounty + %points;
      say(0, 0, getName(%attackerPlayer) @ " neutralized a Cybrid ($" @ %points @ " bounty).");
    }

    $MItargets--;
    if($MItargets < 1)
    {
      $MIstateOfMission = 2;
      say(0, 0, "Mission complete.");
      Mercs::endMission();
    }
  }

  if($MIinGamePlayers < 1)
  {
    $MIstateOfMission = 3;
    say(0, 0, "Mission failed!");
    $MItotalPay = 0;
    Mercs::endMission();
  }
}



//===================


function Mercs::addMoney(%player, %money)
{
  %player.MImoney = %player.MImoney + %money;
  if(%player.MImoney < 0)
  {
    %player.MIdeficit = %player.MImoney;
    %player.MImoney = 0;
  }
  $MIPlayerMoney[%player.MIid] = %player.MImoney;  
  Mercs::InscribePlayers();
}


function Mercs::endMission()
{
  %count = playerManager::getPlayerCount();
  if(%count != 0) Mercs::payouts();

  for(%i=0;%i<$MItargets;%i++)
  {
    schedule("deleteObject(" @ $MItarget[%i] @ ");", 1);
  }

  while(%garbage != 0)
  {
    %garbage = getNextObject("MissionCleanup", 0);
    deleteObject(%garbage);
  }

  schedule("missionEndConditionMet();", 10);
}


function Mercs::inscribePlayers()
{
  $MIplaceHolder = "Mercenaries, Inc. Player List";
  export("$MIplaceHolder", "multiplayer\\MercsIncPlayerList.cs", overwrite);
  fileWrite( "multiplayer\\MercsIncPlayerList.cs" , append, "");
  fileWrite( "multiplayer\\MercsIncPlayerList.cs" , append, "$MIPlayers = " @ $MIPlayers @ ";");

  for(%i=0; %i<$MIPlayers; %i++)
  {
    fileWrite( "multiplayer\\MercsIncPlayerList.cs" , append, "$MIPlayer[" @ %i @ "] = \"" @ $MIPlayer[%i] @ "\";");
    fileWrite( "multiplayer\\MercsIncPlayerList.cs" , append, "$MIPlayerMoney[" @ %i @ "] = " @ $MIPlayerMoney[%i] @ ";");
    fileWrite( "multiplayer\\MercsIncPlayerList.cs" , append, "$MIPlayerPassword[" @ %i @ "] = \"" @ $MIPlayerPassword[%i] @ "\";");
  }
}


function Mercs::keepVehicleShutdown(%vehicle)
{
  if($MIstateOfMission != 0)
  {
    order(%vehicle, shutdown, false);
    return;
  }

  if(!isShutdown(%vehicle)) order(%vehicle, shutdown, true);
  schedule("Mercs::keepVehicleShutdown(" @ %vehicle @ ", true);", 1);
}


function Mercs::payouts()
{
  %missionPay = $MItotalPay;
  %count = playerManager::getPlayerCount();

  %mercCount = 0;
  for(%i = 0; %i < %count; %i++)
  {
    %player = playerManager::getPlayerNum(%i);
    if(%player.MIloggedIn == 1) %mercCount++;
  }
  if(%mercCount != 1) %mercPlural = "s ";
   else %mercPlural = "";

  %missionPayPerPlayer = Mercs::round(%missionPay / %mercCount);

  for(%i = 0; %i < %count; %i++)
  {
    %player = playerManager::getPlayerNum(%i);
    %player.MIstartMission = 0;

    %playerPayout = 0;
    %playerPayout = %missionPayPerPlayer;
    %playerPayout = %playerPayout + %player.MIbounty;
    %playerPayout = %playerPayout - %player.MItransportCost;
    %playerPayout = %playerPayout + %player.deficit;

    %player.deficit = 0;

    if(%playerPayout < 0) %playerPayout = 0;

    Mercs::addMoney(%player, %playerPayout);

    if($MIstateOfMission == 2) %outCome = "Mission completed\n";
     else if($MIstateOfMission == 3) %outCome = "Mission failed\n\nNo mission pay\n";
    if(%missionPayPerPlayer > 0) %missionNote = "$" @ %missionPay @ " total pay for " @ %mercCount @ " merc" @ %mercPlural @ " = $" @ %missionPayPerPlayer @ " split\n"; 
     else %missionNote = "";
    if(%player.MIbounty > 0) %bountyNote = "+$" @ %player.MIbounty @ " kill bounty\n";
     else %bountyNote = "";
    if(%player.MItransportCost > 0) %transportNote = "-$" @ %player.MItransportCost @ " transport fees\n";
     else %transportNote = "";
    if(%player.MIlostVehicle == 1 && %player.MIvehicleCost > 1 ) %lostVehNote = "-$" @ %player.MIvehicleCost @ " (lost vehicle)\n";
     else %lostVehNote = "";

    messagebox(%player, %outCome @ %missionNote @ %bountyNote @ %transportNote @ %lostVehNote @ "Total payout: $" @ %playerPayout @ 
         "\n Your new account: $" @ %player.MImoney);
  }
}


function Mercs::startCheck()
{
  %ready = 1;
  %waitingFor = 0;

  %count = playerManager::getPlayerCount();

  if(%count == 0)
  {
    Mercs::endMission();
  }

  for(%i = 0; %i < %count; %i++)
  {
    %player = playerManager::getPlayerNum(%i);
    if(%player.MIstartMission != 1)
    {
      %ready = 0;
      %waitingFor++;
    }
  }

  if(%ready == 0)
  {
    if(%waitingFor != 1) %plural = "s.";
     else %plural = ".";

    say(0, 0, "Waiting for " @ %waitingFor @ " player" @ %plural );
  }

  else
  {
    say(0, 0, "All pilots have logged in.  Initiating mission.");
    Mercs::startMission();
  }
}


function Mercs::startMission()                 // Testing
{
  exec("MercsIncUnitList.cs");

  $MIstateOfMission = 1;

  %count = playerManager::getPlayerCount();
  for(%i = 0; %i < %count; %i++)
  {
    %player = playerManager::getPlayerNum(%i);
    %vehicle = playerManager::playerNumToVehicleID(%player);
  }

  if($MImissionParameter[0] == 1) $MItargets = randomInt(1,2);
  if($MImissionParameter[0] == 2) $MItargets = randomInt(3,4);

  echo("MITargets: " @ $MITargets);
  
  %vehiclesToAllocate = $MItargets;
  %vehCount = 0;
  %marker = newObject("Marker", Simtrigger);
  while(%vehiclesToAllocate > 0)
  {
    %distance = 0;
    %player = playerManager::getPlayerNum(0);
    %vehicle = playerManager::playerNumToVehicleId(%player);
    %x = getPosition(%vehicle, x);
    %y = getPosition(%vehicle, y);

    while(%distance < 1000 || %distance > 1500)
    {
      %x2 = randomInt(%x - 3000, %x + 3000);
      %y2 = randomInt(%y - 3000, %y + 3000);

      %xsqr = (%x - %x2) * (%x - %x2);
      %ysqr = (%y - %y2) * (%y - %y2);

      %distance = sqrt( %xsqr + %ysqr);
    }
    %z2 = getTerrainHeight(%x2, %y2);
    setPosition(%marker, %x2, %y2, %z2);    

  	%dx = (%x2 - %x) / %distance;
  	%dy = (%y2 - %y) / %distance;
  	if(%dy >= 0.866) %dir = "north";
  	if(((%dy < 0.866) && (%dy > 0)) && ((%dx < 0.866) && (%dx > 0)))
  		%dir = "northeast";
  	if(%dx >= 0.866) %dir = "east";
  	if(((%dy > -0.866) && (%dy < 0)) && ((%dx < 0.866) && (%dx > 0)))
  		%dir = "southeast";
  	if(%dy <= -0.866) %dir = "south";
  	if(((%dy > -0.866) && (%dy < 0)) && ((%dx > -0.866) && (%dx < 0)))
  		%dir = "southwest";
  	if(%dx <= -0.866) %dir = "west";
  	if(((%dy < 0.866) && (%dy > 0)) && ((%dx > -0.866) && (%dx < 0)))
  		%dir = "northwest";
  	%distance = floor(%distance+0.5);
//  	Say(0,0,"TACCOM: Radar reports an infestation "@ %distance @"m to the "@%dir@".");
 
    %vehiclesInBattleGroup = RandomInt(1, 6);
    if(%vehiclesInBattleGroup > %vehiclesToAllocate) %vehiclesInBattleGroup = %vehiclesToAllocate;

    for(%i=0; %i<%vehiclesInBattleGroup; %i++)
    {
      if($MImissionParameter[1] == 1)
      {
         echo("vehiclesInBattleGroup = " @ %vehiclesInBattleGroup);
         echo("vehiclesToAllocate = " @ %vehiclesToAllocate);
         %i = randomInt(1, %MIAIVehicleCount[1]) - 1;
         %herc= loadObject("AIVehicle", %MIAIVehicle[1, %i] ); 
      }
  
      setTeam( %herc, *IDSTR_TEAM_PURPLE );
      setPilotId(%herc,28);
      %vehiclesToAllocate--;
  
      %distance2 = 0;
      %x3 = getPosition(%marker, x);
      %y3 = getPosition(%marker, y);
  
      while(%distance2 < 50 || %distance2 > 150)
      {
        %x4 = randomInt(%x3 - 200, %x3 + 200);
        %y4 = randomInt(%y3 - 200, %y3 + 200);
  
        %xsqr2 = (%x3 - %x4) * (%x3 - %x4);
        %ysqr2 = (%y3 - %y4) * (%y3 - %y4);
  
        %distance2 = sqrt( %xsqr2 + %ysqr2);
      }
      %z4 = getTerrainHeight(%x4, %y4);
      setPosition(%herc, %x4, %y4, %z4);    

  		addToSet( "MissionCleanup", %herc );
      order( %herc, holdfire, true );
      schedule( " order( " @ %herc @ ", holdfire, false);", 4 );
      $MItarget[%vehCount] = %herc;
      %vehCount++;
      if(%vehiclesToAllocate < 1)
      {
         deleteObject(%marker);
         break;  
      }  
    }
  }
}

//===================


function remoteMercs::Login(%player, %password)
{
  if(%player.MIstartMission == 1) return;

  if(%player.MIloggedIn != 1)
  {
    messagebox(%player, "Sorry, this mission has already launched.  Please wait until the battlegroup returns to the lobby.  Thank you for your patience.");
  }
  else
  {
    $MIInGamePlayers++;
    %player.MIstartMission = 1;
    %player.MIbounty = 0;
    %player.MIlostVehicle = 0;
    say(0, 0, getName(%player) @ " has logged in.");
    Mercs::startCheck();
  }
}


//===================


function remoteMercs::statusReport(%player)
{
  %testObject = getNextObject("MissionCleanup", 0);
  if(getObjectName(%testObject) != "AIVehicle")
  {
    %testObject = getNextObject("MissionCleanup", %testObject);
    if(%testObject == 0) say(%player, 0, "No targets detected.");
    if(getObjectName(%testObject) == "AIVehicle")
    {
      %vehicleID = playerManager::playerNumToVehicleID(%player);
      %x = getPosition(%vehicleID, x);
      %y = getPosition(%vehicleID, y);

      %target = getObjectID(%testObject);
      %x2 = getPosition(%target, x);
      %y2 = getPosition(%target, y);

      %distance = getDistance(%vehicleID, %target);

    	%dx = (%x2 - %x) / %distance;
    	%dy = (%y2 - %y) / %distance;
    	if(%dy >= 0.866) %dir = "north";
    	if(((%dy < 0.866) && (%dy > 0)) && ((%dx < 0.866) && (%dx > 0)))
    		%dir = "northeast";
    	if(%dx >= 0.866) %dir = "east";
    	if(((%dy > -0.866) && (%dy < 0)) && ((%dx < 0.866) && (%dx > 0)))
    		%dir = "southeast";
    	if(%dy <= -0.866) %dir = "south";
    	if(((%dy > -0.866) && (%dy < 0)) && ((%dx > -0.866) && (%dx < 0)))
    		%dir = "southwest";
    	if(%dx <= -0.866) %dir = "west";
    	if(((%dy < 0.866) && (%dy > 0)) && ((%dx > -0.866) && (%dx < 0)))
    		%dir = "northwest";
    	%distance = floor(%distance+0.5);
  	Say(0,0,"TACCOM: Target detected "@ %distance @"m "@%dir@" from your position.");
    }
  }
 
  else
  {
    %vehicleID = playerManager::playerNumToVehicleID(%player);
    %x = getPosition(%vehicleID, x);
    %y = getPosition(%vehicleID, y);

    %target = getObjectID(%testObject);
    %x2 = getPosition(%target, x);
    %y2 = getPosition(%target, y);

    %distance = getDistance(%vehicleID, %target);

  	%dx = (%x2 - %x) / %distance;
  	%dy = (%y2 - %y) / %distance;
  	if(%dy >= 0.866) %dir = "north";
  	if(((%dy < 0.866) && (%dy > 0)) && ((%dx < 0.866) && (%dx > 0)))
  		%dir = "northeast";
  	if(%dx >= 0.866) %dir = "east";
  	if(((%dy > -0.866) && (%dy < 0)) && ((%dx < 0.866) && (%dx > 0)))
  		%dir = "southeast";
  	if(%dy <= -0.866) %dir = "south";
  	if(((%dy > -0.866) && (%dy < 0)) && ((%dx > -0.866) && (%dx < 0)))
  		%dir = "southwest";
  	if(%dx <= -0.866) %dir = "west";
  	if(((%dy < 0.866) && (%dy > 0)) && ((%dx > -0.866) && (%dx < 0)))
  		%dir = "northwest";
  	%distance = floor(%distance+0.5);
  	Say(0,0,"TACCOM: Target detected "@ %distance @"m "@%dir@" from your position.");
  }
}



function Mercs::round(%number)
{
  %roundNum = %number;
  if(%number > 0)
  {
    %excess = (%number - floor(%number));
    if(%excess >= 0.5)
    {
      %roundNum = (floor(%number) + 1);
    }
    else
    {
      %roundNum = floor(%number);
    }
  }
  if(%number < 0)
  {
    %excess = (floor(%number) - %number);
    if(%excess <= 0.5)
    {
      %roundNum = floor(%number);
    }
    else
    {
      %roundNum = (floor(%number) - 1);
    }
   }
   return %roundNum;
}


Pilot Weak
{
   id = 28;
   
   skill = 0.1;
   accuracy = 0.1;
   aggressiveness = 0.8;
   activateDist = 1200.0;
   deactivateBuff = 200.0;
   targetFreq = 2.0;
   trackFreq = 2.3;
   fireFreq = 2.0;
   LOSFreq = 1.2;
   orderFreq = 8.0;
};
